# -*- coding: utf-8 -*-

# Copyright 2015-2018 Martijn van Exel.
# This file is part of the overpass-api-python-wrapper project
# which is licensed under Apache 2.0.
# See LICENSE.txt for the full license text.


class MapQuery(object):
    """Query to retrieve complete ways and relations in an area."""

    _QUERY_TEMPLATE = "(node({south},{west},{north},{east});<;>;);"

    def __init__(self, south, west, north, east):
        """
        Initialize query with given bounding box.

        :param bbox Bounding box with limit values in format west, south,
        east, north.
        """
        self.west = west
        self.south = south
        self.east = east
        self.north = north

    def __str__(self):
        return self._QUERY_TEMPLATE.format(
            west=self.west, south=self.south, east=self.east, north=self.north
        )


class WayQuery(object):
    """Query to retrieve a set of ways and their dependent nodes satisfying the input parameters."""

    _QUERY_TEMPLATE = "(way{query_parameters});(._;>;);"

    def __init__(self, query_parameters):
        """Initialize a query for a set of ways satisfying the given parameters.

        :param query_parameters Overpass QL query parameters
        """
        self.query_parameters = query_parameters

    def __str__(self):
        return self._QUERY_TEMPLATE.format(query_parameters=self.query_parameters)



# -*- coding: utf-8 -*-

# Copyright 2015-2018 Martijn van Exel.
# This file is part of the overpass-api-python-wrapper project
# which is licensed under Apache 2.0.
# See LICENSE.txt for the full license text.

"""Thin wrapper around the OpenStreetMap Overpass API."""

__title__ = "overpass"
__version__ = "0.7"
__license__ = "Apache 2.0"

from .api import API
from .queries import MapQuery, WayQuery
from .errors import (
    OverpassError,
    OverpassSyntaxError,
    TimeoutError,
    MultipleRequestsError,
    ServerLoadError,
    UnknownOverpassError,
)
from .utils import *



#!/usr/bin/env python

# Copyright 2015-2018 Martijn van Exel.
# This file is part of the overpass-api-python-wrapper project
# which is licensed under Apache 2.0.
# See LICENSE.txt for the full license text.

# Retrieves a list of unique usernames and user IDs for a named area.

import overpass

# Change this to the name of the area you're interested in.
# Keep it small to not abuse the Overpass server.
area_name = "Kanab"

query = """area[name="{}"]->.a;(node(area.a);<;);""".format(area_name)
users = {"ids": [], "usernames": []}
message_urls = []
api = overpass.API(debug=False)
result = api.Get(query, responseformat="csv(::uid,::user)", verbosity="meta")
del result[0]  # header
for row in result:
    uid = int(row[0])
    username = row[1]
    if uid in users["ids"]:
        continue
    users["ids"].append(uid)
    users["usernames"].append(username)
    message_urls.append("https://www.openstreetmap.org/message/new/{}".format(username))
print(users)
print(message_urls)



# Copyright 2015-2018 Martijn van Exel.
# This file is part of the overpass-api-python-wrapper project
# which is licensed under Apache 2.0.
# See LICENSE.txt for the full license text.

import overpass
import geojson
import pickle
import os


def test_initialize_api():
    api = overpass.API()
    assert isinstance(api, overpass.API)
    assert api.debug is False


def test_geojson():
    api = overpass.API(debug=True)

    map_query = overpass.MapQuery(37.86517, -122.31851, 37.86687, -122.31635)
    osm_geo = api.get(map_query)
    assert len(osm_geo["features"]) > 1

    osm_geo = api.get("node(area:3602758138)[amenity=cafe]")
    assert len(osm_geo["features"]) > 1


def test_geojson_extended():

    class API(overpass.API):
        def _get_from_overpass(self, query):
            return pickle.load(open(os.path.join(os.path.dirname(__file__), "example.response"), "rb"))

    # The commented code should only be executed once when major changes to the Overpass API and/or to this wrapper are
    # introduced. One than has to manually verify that the date in the  example.response file from the Overpass API
    # matches the data in the example.json file generated by this wrapper.
    #
    # The reason for this approach is the following: It is not safe to make calls to the actual API in this test as the
    # API might momentarily be unavailable and the underlying data can also change at any moment. The commented code is
    # needed to create the example.response and example.json files. The example.response file is subsequently used to
    # fake the _get_from_overpass method during the tests and the example.json file is the reference that we are
    # asserting against.
    #
    # api = overpass.API()
    # osm_geo = api.get("rel(6518385);out body geom;way(10322303);out body geom;node(4927326183);", verbosity='body geom')
    # pickle.dump(api._get_from_overpass("[out:json];rel(6518385);out body geom;way(10322303);out body geom;node(4927326183);out body geom;"),
    #             open(os.path.join(os.path.dirname(__file__), "example.response"), "wb"),
    #             protocol=2)
    # geojson.dump(osm_geo, open(os.path.join(os.path.dirname(__file__), "example.json"), "w"))

    api = API()
    osm_geo = api.get("rel(6518385);out body geom;way(10322303);out body geom;node(4927326183);", verbosity='body geom')
    ref_geo = geojson.load(open(os.path.join(os.path.dirname(__file__), "example.json"), "r"))
    assert osm_geo==ref_geo



#!/usr/bin/env python

# Copyright 2015-2018 Martijn van Exel.
# This file is part of the overpass-api-python-wrapper project
# which is licensed under Apache 2.0.
# See LICENSE.txt for the full license text.

"""An as yet not working example of using overpassify."""

import overpass
from overpassify import overpassify

api = overpass.API()


@api.Get
@overpassify
def response():
    Settings(timeout=1400)
    search = Area(3600134503) + Area(3600134502)
    ways = Way(
        search,
        maxspeed=None,
        highway=NotRegex("cycleway|footway|path|service"),
        access=NotRegex("private"),
    )
    out(ways, body=True, geom=True, qt=True)
    noop()


print(response)



#!/usr/bin/env python

# Copyright 2015-2018 Martijn van Exel.
# This file is part of the overpass-api-python-wrapper project
# which is licensed under Apache 2.0.
# See LICENSE.txt for the full license text.

import overpass

api = overpass.API()
response = api.get('node["name"="Salt Lake City"]')
print(
    [(feature["id"], feature["properties"]["name"]) for feature in response["features"]]
)
